@@ -1,5 +1,5 @@
 from jsonutils.base import JSONObject
 from jsonutils.functions.parsers import parse_bool, parse_datetime, parse_float
-from jsonutils.query import All, Q
+from jsonutils.query import All, I, Q
 
 from .release import __version__
