@@ -17,12 +17,13 @@ from jsonutils.exceptions import (
 )
 from jsonutils.functions.parsers import (
     _parse_query,
+    _parse_query_key,
     parse_bool,
     parse_datetime,
     parse_float,
     url_validator,
 )
-from jsonutils.query import ParentList, QuerySet
+from jsonutils.query import All, KeyQuerySet, ParentList, QuerySet
 from jsonutils.utils.dict import UUIDdict, ValuesDict
 from jsonutils.utils.retry import retry_function
 
@@ -93,6 +94,15 @@ class JSONObject:
     This class does not contain any instances of.
     """
 
+    _RESERVED_ATTRIBUTES = (
+        "_key",
+        "_index",
+        "parent",
+        "_id",
+        "_child_objects",
+        "_is_annotation",
+    )
+
     def __new__(cls, data=None, raise_exception=False):
         if not isinstance(raise_exception, bool):
             raise TypeError(
@@ -128,6 +138,7 @@ class JSONObject:
         Open an external JSON file.
         If a valid url string is passed, then it will try to make a get request to such a target and decode a json file
         """
+        file = str(file)
         if url_validator(file):
             req = retry_function(requests.get, file, raise_exception=False)
             try:
@@ -156,6 +167,8 @@ class JSONNode:
         _id: unique uuid4 hex identifier of object
     """
 
+    __odir__ = object.__dir__  # rename old __dir__ method to __odir__
+
     def __init__(self, *args, **kwargs):
 
         self._key = None
@@ -208,11 +221,22 @@ class JSONNode:
             parent = parent.parent
         return last
 
-    def values(self, *keys, search_upwards=True):
+    def values(self, *keys, search_upwards=True, **kwargs):
         output_dict = ValuesDict({k: None for k in keys})
 
+        if kwargs:
+            output_dict.update({k: None for k in kwargs})
+
         for key in keys:
             obj = self
+            if "__" in key:  # traverse by childs in this case
+                child_key = key.split("__")[-1]
+                if not child_key:
+                    raise ValueError("Wrong syntax within query values request")
+                output_dict[key] = obj.parent.get(
+                    **{child_key: All}, native_types_=True, throw_exceptions_=False
+                )
+                continue
             while True:
                 if isinstance(obj, JSONDict):
                     if key in obj:
@@ -224,6 +248,30 @@ class JSONNode:
                         break
                 else:
                     break
+
+        if kwargs:
+            for k, v in kwargs.items():
+                obj = self
+                if "__" in v:  # traverse by childs in this case
+                    child_key = v.split("__")[-1]
+                    if not child_key:
+                        raise ValueError("Wrong syntax within query values request")
+                    output_dict[k] = obj.parent.get(
+                        **{child_key: All}, native_types_=True, throw_exceptions_=False
+                    )
+                    continue
+                while True:
+                    if isinstance(obj, JSONDict):
+                        if v in obj:
+                            output_dict[k] = obj[v]._data
+                            break
+                    if search_upwards:
+                        obj = obj.parent
+                        if obj is None:
+                            break
+                    else:
+                        break
+
         return output_dict
 
     def __str__(self):
@@ -513,11 +561,60 @@ class JSONCompose(JSONNode):
                     ch.annotate(**kwargs)
         return self
 
+    def query_key(
+        self,
+        pattern,
+        recursive_=None,
+        include_parent_=None,
+        stop_at_match_=None,
+        native_types_=None,
+        **q,
+    ):
+        if not isinstance(stop_at_match_, (int, type(None))):
+            raise TypeError(
+                f"Argument stop_at_match_ must be an integer or NoneType, not {type(stop_at_match_)}"
+            )
+
+        # within a particular parent node, each child in _child_objects registry must have a unique key
+
+        # ---- DYNAMIC CONFIG ----
+        if recursive_ is None:
+            recursive_ = config.recursive_queries
+        if include_parent_ is None:
+            include_parent_ = config.include_parents
+        if native_types_ is None:
+            native_types_ = config.native_types
+        # ------------------------
+        queryset = KeyQuerySet()
+        if native_types_:
+            queryset._native_types = True
+        queryset._root = self  # the node which sends the query
+        children = self._child_objects.values()
+        for child in children:
+            # if child satisfies query request, it will be appended to the queryset object
+            check, obj = _parse_query_key(child, pattern, include_parent_, **q)
+            if check:
+                queryset.append(obj)
+                if stop_at_match_ and queryset.count() >= stop_at_match_:
+                    return queryset
+
+            # if child is also a compose object, it will send the same query to its children recursively
+            if child.is_composed and recursive_:
+                queryset += child.query_key(
+                    pattern,
+                    recursive_=recursive_,
+                    include_parent_=include_parent_,
+                    stop_at_match_=stop_at_match_,
+                    native_types_=native_types_,
+                    **q,
+                )
+        return queryset
+
     def _remove_annotations(self, recursive=True):
 
         if isinstance(self, JSONDict):
             for key, value in list(self.items()):
-                if hasattr(value, "_is_annotation"):
+                if "_is_annotation" in value.__odir__():
                     self.pop(key)
                 if value.is_composed and recursive:
                     value._remove_annotations()
@@ -537,10 +634,10 @@ class JSONSingleton(JSONNode):
 
     is_composed = False
 
-    def query(self, recursive_=None, include_parent_=None, stop_at_match_=None, **q):
+    def query(self, **kwargs):
         return QuerySet()
 
-    def get(self, recursive_=None, include_parent_=None, throw_exceptions_=None, **q):
+    def get(self, **kwargs):
         return
 
 
@@ -561,7 +658,7 @@ class JSONDict(dict, JSONCompose):
     def __dir__(self):
         # for autocompletion stuff
         if config.autocomplete_only_nodes:
-            return self.keys()
+            return list(self.keys())
         else:
             return list(self.keys()) + super().__dir__()
 
@@ -569,7 +666,7 @@ class JSONDict(dict, JSONCompose):
         try:
             return self.__getitem__(name)
         except KeyError:  # if a key error is thrown, then it will call __dir__
-            raise AttributeError
+            return
 
     def __setitem__(self, k, v):
         """
@@ -601,16 +698,7 @@ class JSONDict(dict, JSONCompose):
     def __setattr__(self, name, value):
         """To define behaviour when setting an atributte. It must register a new node if not a reserved keyword"""
 
-        RESERVED_ATTRIBUTES = (
-            "_key",
-            "_index",
-            "parent",
-            "_id",
-            "_child_objects",
-            "_is_annotation",
-        )
-
-        if name in RESERVED_ATTRIBUTES:
+        if name in JSONObject._RESERVED_ATTRIBUTES:
             return super().__setattr__(name, value)
         else:
             return self.__setitem__(name, value)
@@ -692,7 +780,7 @@ class JSONList(list, JSONCompose):
     def __setattr__(self, name, value):
         """To define behaviour when setting an atributte. It must register a new node if not a reserved keyword"""
 
-        if name in ("_key", "_index", "parent", "_id", "_child_objects"):
+        if name in JSONObject._RESERVED_ATTRIBUTES:
             return super().__setattr__(name, value)
         else:
             name = int(name.replace("_", ""))
@@ -777,6 +865,17 @@ class JSONStr(str, JSONSingleton):
 
         return parse_datetime(self, **kwargs)
 
+    def to_timestamp(self, **kwargs):
+        """Try to parse a POSIX timestamp string from self string"""
+
+        datetime_ = parse_datetime(self, **kwargs)
+        try:
+            result = datetime_.timestamp() * 1000
+        except Exception:
+            return
+        else:
+            return result
+
     def to_bool(self):
         """Trye to parse a bool object from self string."""
 
