@@ -398,7 +398,12 @@ def _isnull(node, requested_value):
         raise JSONQueryException(
             f"Requested value must be a boolean, not {type(requested_value)}"
         )
-    return bool(node) != requested_value if node != 0 else False
+
+    boolean_value = (
+        True if isinstance(node, (JSONBool, JSONInt, JSONFloat)) else bool(node)
+    )
+
+    return boolean_value != requested_value
 
 
 def _length(node, requested_value):
@@ -438,6 +443,7 @@ def _type(node, requested_value):
         "int",
         "bool",
         "None",
+        "singleton",
     ):
         raise JSONQueryException(
             f"Requested value must be a valid type, not {requested_value}"
@@ -446,15 +452,59 @@ def _type(node, requested_value):
     if isinstance(node, JSONBool):
         if requested_value in (bool, "bool"):
             return True
+        elif requested_value == "singleton":
+            return True
         else:
             return False
     elif isinstance(node, JSONNull):
         if requested_value in (None, "None"):
             return True
+        elif requested_value == "singleton":
+            return True
+        else:
+            return False
+    elif isinstance(node, JSONDict):
+        if requested_value in (dict, "dict"):
+            return True
+        elif requested_value == "singleton":
+            return False
+        else:
+            return False
+    elif isinstance(node, JSONList):
+        if requested_value in (list, "list"):
+            return True
+        elif requested_value == "singleton":
+            return False
+        else:
+            return False
+    elif isinstance(node, JSONFloat):
+        if requested_value in (float, "float"):
+            return True
+        elif requested_value == "singleton":
+            return True
+        else:
+            return False
+    elif isinstance(node, JSONInt):
+        if requested_value in (int, "int"):
+            return True
+        elif requested_value == "singleton":
+            return True
+        else:
+            return False
+    elif isinstance(node, JSONStr):
+        if requested_value in (str, "str"):
+            return True
+        elif requested_value == "singleton":
+            return True
+        elif requested_value in (datetime, "datetime"):
+            if node.to_datetime(only_check=True):
+                return True
+            else:
+                return False
         else:
             return False
     else:
-        return isinstance(node, requested_value)
+        return False
 
 
 def _key(node, requested_value):
@@ -495,3 +545,19 @@ def _path(node, requested_value):
         raise TypeError(
             f"Argument requested_value must be an iterable, str or bool, not {type(requested_value)}"
         )
+
+
+def _notpath(node, requested_value):
+    """
+    This method analyzes whether a given JSONObject does not come from selected path
+    """
+    # TODO store jsonpath in cache to avoid call it repeatdly
+    # TODO add test
+    if isinstance(requested_value, (list, tuple, set)):
+        return not any(i in node.jsonpath.keys for i in requested_value)
+    elif isinstance(requested_value, (str, int)):
+        return requested_value not in node.jsonpath.keys
+    else:
+        raise TypeError(
+            f"Argument requested_value must be an iterable, str or bool, not {type(requested_value)}"
+        )
