@@ -7,6 +7,7 @@ import jsonutils.base as base
 import jsonutils.config as config
 import jsonutils.functions.parsers as parsers
 from jsonutils.exceptions import JSONQueryException
+from jsonutils.utils.dict import ValuesDict
 
 
 class SingleQuery:
@@ -200,12 +201,17 @@ class ValuesList(list):
         self._root = None
 
     def first(self):
-        result = self.__getitem__(0)
-        return result if self.__len__() > 0 else None
 
-    def last(self, native_types_=None):
-        result = self.__getitem__(-1)
-        return result if self.__len__() > 0 else None
+        try:
+            return self.__getitem__(0)
+        except IndexError:
+            return
+
+    def last(self):
+        try:
+            return self.__getitem__(-1)
+        except IndexError:
+            return
 
     def exists(self):
         return True if self.__len__() > 0 else False
@@ -213,7 +219,7 @@ class ValuesList(list):
     def count(self):
         return self.__len__()
 
-    def unique(self):
+    def distinct(self):
         """
         Returns unique values
         """
@@ -271,7 +277,7 @@ class QuerySet(list):
 
     def update(self, new_obj):
         """
-        Update elements of queryset within JSONObject from which they are derived (self._root)
+        Update elements of queryset (inplace) within JSONObject from which they are derived (self._root)
         """
 
         # TODO add test for update when callables
@@ -287,7 +293,7 @@ class QuerySet(list):
             else:
                 exec(f"self._root{path} = new_obj")
 
-    def unique(self, transform=None):
+    def distinct(self, transform=None):
         """
         Returns unique values in a querylist
         Arguments:
@@ -299,6 +305,7 @@ class QuerySet(list):
         # TODO dict option for counting unique values
         unique_values = QuerySet()
         unique_values._root = self._root
+        unique_values._native_types = self._native_types
 
         transformed_values = set()
 
@@ -323,12 +330,14 @@ class QuerySet(list):
         # TODO add test
         output = QuerySet()
         output._root = self._root
+        output._native_types = self._native_types
+
         for item in self:
             if item.parent.query(**q).exists():
                 output.append(item)
         return output
 
-    def values(self, *keys, search_upwards=True):
+    def values(self, *keys, search_upwards=True, **kwargs):
         """
         This method changes the values returned by the queryset.
         The returned values will be python types.
@@ -341,9 +350,18 @@ class QuerySet(list):
         # TODO call node values method for each item
 
         values_list = ValuesList(
-            (item.values(*keys, search_upwards=True) for item in self)
+            (
+                item.values(*keys, search_upwards=search_upwards, **kwargs)
+                for item in self
+            )
         )
         values_list._root = self._root
+        if not values_list:
+            output_dict = ValuesDict({k: None for k in keys})
+
+            if kwargs:
+                output_dict.update({k: None for k in kwargs})
+            values_list.append(output_dict)
         return values_list
 
     def order_by(self, key):
@@ -364,6 +382,7 @@ class QuerySet(list):
             return self
 
         result._root = self._root
+        result._native_types = self._native_types
         return result
 
     # ---- GROUP OPERATIONS ----
@@ -383,6 +402,22 @@ class QuerySet(list):
         return "<QuerySet " + super().__repr__() + ">"
 
 
+class KeyQuerySet(QuerySet):
+    """
+    This is a QuerySet but with more methods for querying keys
+    """
+
+    def keys(self):
+        result = ValuesList()
+        result._root = self._root
+        result._native_types = self._native_types
+
+        for item in self:
+            result.append(item._key)
+
+        return result
+
+
 class AllChoices(type):
     pass
 
@@ -442,3 +477,14 @@ class ExtractYear:
             return str(query_value) in str(self.year)
         else:
             return False
+
+
+class I:
+    """Ignore case Object wrapper, for using in query_key method instead of re.compile the pattern with re.I flag"""
+
+    def __init__(self, pattern):
+        if not isinstance(pattern, str):
+            raise TypeError(
+                f"Argument pattern must be an str instance, not {type(pattern)}"
+            )
+        self.data = re.compile(pattern, re.I)
