@@ -1,6 +1,8 @@
 import json
+import re
 import unittest
 from datetime import datetime
+from pathlib import Path
 
 import jsonutils as js
 import pytz
@@ -20,7 +22,9 @@ from jsonutils.base import (
 )
 from jsonutils.encoders import JSONObjectEncoder
 from jsonutils.exceptions import JSONQueryException, JSONQueryMultipleValues
-from jsonutils.query import All, QuerySet, SingleQuery
+from jsonutils.query import All, ExtractYear, QuerySet, SingleQuery
+
+BASE_PATH = Path(js.__file__).parent.resolve()
 
 
 class JsonTest(unittest.TestCase):
@@ -343,6 +347,8 @@ class JsonTest(unittest.TestCase):
 
     def test_queries(self):
 
+        test = JSONObject.open(BASE_PATH / "tests/balance-sheet-example-test.json")
+
         self.assertEqual(JSONObject(dict(A=True)).query(A__isnull=True), [])
         self.assertEqual(JSONObject(dict(A=True)).query(A__isnull=False), [True])
         self.assertEqual(JSONObject(dict(A=False)).query(A__isnull=True), [])
@@ -459,6 +465,18 @@ class JsonTest(unittest.TestCase):
         self.assertEqual(self.test8.query(id__isnull=True), QuerySet())
         self.assertEqual(self.test6.query(data__1=True), QuerySet([[False, True]]))
 
+        self.assertListEqual(
+            test.query(
+                filing_date__isnull=False, filing_date__notpath="all_company_filings"
+            )
+            .order_by("-filing_date")
+            .values(date="filing_date", form_type="form_type", url="form_url"),
+            [
+                {"date": "2017-09-27", "form_type": "1-A/A", "url": None},
+                {"date": "2017-09-06", "form_type": "1-A", "url": None},
+            ],
+        )
+
     def test_multiqueries(self):
         test = JSONObject(
             [
@@ -772,7 +790,7 @@ class JsonTest(unittest.TestCase):
 
         self.assertSetEqual(set(test.data._0._child_objects.values()), {30})
         self.assertFalse(test.query(name=All).exists())
-        self.assertRaises(AttributeError, lambda: test.data._0.name)
+        self.assertIsNone(test.data._0.name)
 
     def test_multiparent(self):
         test = JSONObject(
@@ -902,6 +920,18 @@ class JsonTest(unittest.TestCase):
             test.get(field1=1, field1__parents__c_type="C").values("date", "name"),
             {"date": "2021-01-01", "name": "NAME"},
         )
+        self.assertEqual(
+            test.get(field1=1, field1__parents__c_type="C").values(
+                datetime="date", new_name="name"
+            ),
+            {"datetime": "2021-01-01", "new_name": "NAME"},
+        )
+        self.assertEqual(
+            test.get(field1=1, field1__parents__c_type="C").values(
+                "date", new_name="name"
+            ),
+            {"date": "2021-01-01", "new_name": "NAME"},
+        )
         self.assertEqual(
             test.get(
                 field1__parents__c_date__year=2022, field1__notpath="filing"
@@ -926,3 +956,52 @@ class JsonTest(unittest.TestCase):
         self.assertIsNone(
             test2.get(fake=True).values("timestamp", "age").timestamp,
         )
+
+    def test_distinct(self):
+
+        self.assertEqual(
+            self.test6.query(timestamp__year=2021)
+            .order_by("-timestamp")
+            .distinct(lambda x: ExtractYear(x).year),
+            ["2021-05-05 18:00:25"],
+        )
+
+    def test_query_keys(self):
+
+        test = JSONObject({"name": {"NAME": "Dan", "name": True}})
+
+        self.assertListEqual(
+            test.query_key(".*ame", exact=All), [{"NAME": "Dan", "name": True}, True]
+        )
+        self.assertListEqual(
+            test.query_key(re.compile(".*AmE", re.I), exact=All),
+            [{"NAME": "Dan", "name": True}, "Dan", True],
+        )
+        self.assertListEqual(
+            test.query_key(re.compile(".*AmE", re.I), type__=bool),
+            QuerySet([True]),
+        )
+        self.assertListEqual(
+            test.query_key(
+                re.compile(".*AmE", re.I),
+                type__=bool,
+                parent={"NAME": "Dan", "name": True},
+            ),
+            QuerySet([True]),
+        )
+        self.assertListEqual(
+            test.query_key(
+                re.compile(".*AmE", re.I),
+                type__=bool,
+                parent__key="name",
+            ),
+            QuerySet([True]),
+        )
+        self.assertListEqual(
+            test.query_key(
+                re.compile(".*AmE", re.I),
+                type__=bool,
+                path="fake",
+            ),
+            QuerySet([]),
+        )
